INTEGER          = 'INTEGER'  
FLOAT            = 'FLOAT'  
PLUS             = 'PLUS'  
MINUS            = 'MINUS'  
MUL              = 'MUL'  
DIV              = 'DIV'  
LPAREN           = 'LPAREN'  
RPAREN           = 'RPAREN'  
IDENTIFIER       = 'IDENTIFIER'  
ASSIGN           = 'ASSIGN'  
SEMI             = 'SEMI'      # Semicolon  
EOF              = 'EOF'       # End-of-file  
INPUT            = 'INPUT'     # van muji  
PRINT            = 'PRINT'     # her muji  
VAR_DECL         = 'VAR_DECL'  # dey muji  
IF               = 'IF'        # yedi muji  
ELIF             = 'ELIF'      # natra bhane muji  
ELSE             = 'ELSE'      # natra muji  
PROGRAM_START    = 'PROGRAM_START'  # oe muji  
PROGRAM_END      = 'PROGRAM_END'    # lala muji  
LBRACE           = 'LBRACE'  
RBRACE           = 'RBRACE'  
EQ               = 'EQ'        # ==  
NE               = 'NE'        # !=  
LT               = 'LT'        # <  
GT               = 'GT'        # >  
LTE              = 'LTE'       # <=  
GTE              = 'GTE'       # >=  
STRING           = 'STRING'  
FOR              = 'FOR'  
WHILE            = 'WHILE'  